﻿using System;
using System.Threading.Tasks;

namespace HeuristicLab.Algorithms.PushGP.Cli {
  using HeuristicLab.Problems.ProgramSynthesis.Push.Configuration;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter;

  class Program {
    static void Main(string[] args) {
      if (args.Length == 0) return;

      var code = args[0];
      EvaluateStepwise(code).Wait();

      Console.ReadKey();
    }

    static async Task EvaluateStepwise(string code) {
      var interpreter = new PushInterpreter(new PushConfiguration {
        TopLevelPushCode = false
      });

      interpreter.RunAsync(code, true).Wait();

      while (!interpreter.IsCompleted) {
        Console.Clear();
        interpreter.PrintStacks();
        interpreter.Step();

        var input = Console.ReadKey();
        if (input.Key == ConsoleKey.Escape) {
          break;
        } else if (input.Key == ConsoleKey.Spacebar) {
          await interpreter.ResumeAsync();
        }
      }

      Console.Clear();
      interpreter.PrintStacks();
    }
  }
}